// Copyright © 2024 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/wait.h>
#include <pthread.h>
#include <essdk/es_sys_memory.h>
#include <essdk/es_vb_memory.h>

#define ES_MEM_INFO_ON
#ifdef ES_MEM_INFO_ON
#define ES_MEM_ERR(format, ...) printf("%s %s:%5d " format, "[E][MEM_TEST] ", __FUNCTION__, __LINE__, ##__VA_ARGS__)
#define ES_MEM_INFO(format, ...) printf("%s %s:%5d " format, "[I][MEM_TEST] ", __FUNCTION__, __LINE__, ##__VA_ARGS__)
#endif
#ifndef ES_MEM_ERR
#define ES_MEM_ERR(format, ...)
#endif
#ifndef ES_MEM_INFO
#define ES_MEM_INFO(format, ...)
#endif

#define LEN_4K 4 * 1024
#define LEN_8K 8 * 1024
#define LEN_256K 256 * 1024
#define LEN_512K 512 * 1024
#define LEN_1M 1 * 1024 * 1024
#define LEN_2M 2 * 1024 * 1024
#define LEN_3M 3 * 1024 * 1024
#define LEN_4M 4 * 1024 * 1024
#define LEN_5M 5 * 1024 * 1024
#define LEN_8M 8 * 1024 * 1024
#define LEN_10M 10 * 1024 * 1024

#define DEFAULT_MMZ_NAME "mmz_nid_0_part_0"

static inline ES_S32 hexval(ES_CHAR c)
{
    switch (c) {
        case '0' ... '9':
            c -= '0';
            break;

        case 'a' ... 'f':
            c -= 'a';
            c += 10;
            break;

        case 'A' ... 'F':
            c -= 'A';
            c += 10;
            break;
    }
    return c;
}

ES_S32 str2int(ES_CHAR *str)
{
    ES_S32 val = 0;
    ES_CHAR *p, *cp = str;
    ES_S32 base = 10, neg = 1;

    if ((p = strstr(cp, "0x")) != NULL) {
        cp = p + 2;
        base = 16;
    }

    if ((p = strstr(cp, "-")) != NULL) {
        cp = p + 1;
        base = 10;
        neg = -1;
    }

    for (; *cp != 0;) {
        val *= base;
        val += hexval(*cp);
        cp++;
    }

    return val * neg;
}

struct cache_info {
    const ES_CHAR *zone;
    ES_S32 kernel_cache_type;
    ES_S32 mmap_cache_type;
};
struct comm_case_info {
    struct cache_info info;
};

struct comm_case_info comm_case_data[] = {
    /*src type:		               vmap				mmap */
    {{"system", SYS_CACHE_MODE_CACHED, SYS_CACHE_MODE_CACHED}},            // 0
    {{"linux,cma", SYS_CACHE_MODE_CACHED, SYS_CACHE_MODE_CACHED}},         // 1
    {{"mmz_nid_0_part_0", SYS_CACHE_MODE_CACHED, SYS_CACHE_MODE_NOCACHE}},  // 2
    {{"llc_spram0", SYS_CACHE_MODE_NOCACHE, SYS_CACHE_MODE_NOCACHE}},  // 3
};

ES_S32 comm_mem_case(void)
{
    ES_S32 ret;
    ES_U64 memFd;
    ES_VOID *pVirAddr;
    struct cache_info *info;
    ES_U64 mem_len;
    ES_S32 item_num;
    ES_S32 nid;

    mem_len = LEN_4K;

    for (item_num = 0; item_num < sizeof(comm_case_data) / sizeof(struct comm_case_info); item_num++) {
        ret = 0;
        memFd = 0;
        pVirAddr = NULL;
        info = NULL;

        ES_MEM_INFO("=====================sys mem test mem_len:0x%llx item_num:%d===============\n", mem_len, item_num);
        info = &comm_case_data[item_num].info;

        ret = ES_SYS_MemAlloc(&memFd, info->kernel_cache_type, "sys", info->zone, mem_len);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_SYS_MemAlloc failed, ret %x!\n", ret);
            goto release;
        }

        ret = ES_SYS_RetrieveMemNodeByFd(memFd, &nid);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_SYS_RetrieveMemNodeByFd failed, ret %x!\n", ret);
            goto release;
        }
        ES_MEM_INFO("fd 0x%llx mem node is:%d\n", memFd, nid);
        pVirAddr = ES_SYS_Mmap(memFd, mem_len, info->mmap_cache_type);
        if (!pVirAddr) {
            ES_MEM_ERR("ES_SYS_Mmap failed!\n");
            goto release;
        }

        ret = ES_SYS_RetrieveMemNodeByVaddr(pVirAddr, &nid);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_SYS_RetrieveMemNodeByVaddr failed, ret %x!\n", ret);
        } else
            ES_MEM_INFO("pVirAddr 0x%px mem node is:%d\n", pVirAddr, nid);

        memset(pVirAddr, 0x55, mem_len);

        if (info->mmap_cache_type == SYS_CACHE_MODE_CACHED) {
            ES_MEM_INFO("flush cache!\n");
            ret = ES_SYS_MemFlushCache(memFd);
            if (ES_SUCCESS != ret) {
                ES_MEM_ERR("ES_SYS_MemFlushCache failed, ret %x!\n", ret);
                goto release;
            }
        }

release:
        if (pVirAddr) {
            ret = ES_SYS_Munmap(pVirAddr, mem_len);
            if (ES_SUCCESS != ret) {
                ES_MEM_ERR("ES_SYS_Munmap failed, ret %x!\n", ret);
            }
        }

        if (memFd) {
            ES_MEM_INFO("free memFd(%llx)\n", memFd);
            ret = ES_SYS_MemFree(memFd);
            if (ES_SUCCESS != ret) {
                ES_MEM_ERR("ES_SYS_MemFree failed, ret %x!\n", ret);
            }
        }
    }

    return 0;
}

struct case_vb_config_param_t {
    ES_S32 poolCnt;
    ES_S32 blkSize;
    ES_S32 blkCnt;
    SYS_CACHE_MODE_E enRemapMode;
    ES_CHAR mmzName[ES_MAX_MMZ_NAME_LEN];
};

struct case_vb_config_param_t vb_normal_cases[] = {
    {
        .poolCnt = 1,
        .blkSize = 4096,
        .blkCnt = 1,
        .enRemapMode = SYS_CACHE_MODE_NOCACHE,
        .mmzName = "mmz_nid_0_part_0",
    },
    {
        .poolCnt = ES_VB_MAX_MOD_POOL,
        .blkSize = 4 * 1024,
        .blkCnt = 64,
        .enRemapMode = SYS_CACHE_MODE_CACHED,
        .mmzName = "mmz_nid_0_part_0",
    },
};

ES_S32 VB_mem_case(void)
{
    ES_S32 i = 0, j = 0;
    VB_CONFIG_S vbConfig_in;
    VB_CONFIG_S vbConfig_out;
    VB_POOL_CONFIG_S *poolcfg;
    ES_S32 ret;
    ES_S32 blkSize;
    ES_U64 memFd;
    ES_VOID *pVirAddr;

    for (i = 0; i < sizeof(vb_normal_cases) / sizeof(struct case_vb_config_param_t); i++) {
        memset(&vbConfig_in, 0, sizeof(VB_CONFIG_S));
        memset(&vbConfig_out, 0, sizeof(VB_CONFIG_S));

        vbConfig_in.poolCnt = vb_normal_cases[i].poolCnt;
        blkSize = vb_normal_cases[i].blkSize;

        /*init pool config*/
        for (j = 0; j < vbConfig_in.poolCnt; j++) {
            poolcfg = &vbConfig_in.poolCfgs[j];
            poolcfg->blkSize = vb_normal_cases[i].blkSize;
            poolcfg->blkCnt = vb_normal_cases[i].blkCnt;
            poolcfg->enRemapMode = vb_normal_cases[i].enRemapMode;
            memcpy(poolcfg->mmzName, vb_normal_cases[i].mmzName, strlen(vb_normal_cases[i].mmzName));
        }

        ret = ES_VB_SetConfig(&vbConfig_in);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("SetConfig, ret %x!\n", ret);
            return ret;
        }

        ret = ES_VB_GetConfig(&vbConfig_out);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("GetConfig, ret %x!\n", ret);
            return ret;
        }

        /*compare configure we set and the configure get from sdk*/
        if ((vbConfig_in.poolCnt != vbConfig_out.poolCnt)) {
            ES_MEM_ERR("error:compare poolcfg vbConfig_in.poolCnt != vbConfig_out.poolCnt!\n");
            return -1;
        }

        for (j = 0; j < vbConfig_in.poolCnt; j++) {
            if ((vbConfig_in.poolCfgs[j].blkCnt != vbConfig_out.poolCfgs[j].blkCnt) ||
                (vbConfig_in.poolCfgs[j].blkSize != vbConfig_out.poolCfgs[j].blkSize) ||
                (vbConfig_in.poolCfgs[j].enRemapMode != vbConfig_out.poolCfgs[j].enRemapMode)) {
                ES_MEM_ERR("error:compare poolcfg failed!\n");
                return -1;
            }

            if (strlen(vbConfig_in.poolCfgs[j].mmzName)) {
                if (memcmp(vbConfig_in.poolCfgs[j].mmzName, vbConfig_out.poolCfgs[j].mmzName, ES_MAX_MMZ_NAME_LEN) !=
                    0) {
                    ES_MEM_ERR("error:compare poolcfg mmzName failed!\n");
                    return -1;
                }
            } else {
                if (memcmp(vbConfig_out.poolCfgs[j].mmzName, DEFAULT_MMZ_NAME, strlen(DEFAULT_MMZ_NAME)) != 0) {
                    ES_MEM_ERR("error:compare poolcfg mmzName(%s) failed!\n", vbConfig_out.poolCfgs[j].mmzName);
                    return -1;
                }
            }
        }

        ret = ES_VB_Init();
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("error:ES_VB_Init, ret %x!\n", ret);
            return ret;
        }

        ret = ES_VB_GetBlock(ES_VB_INVALID_POOLID, blkSize, vb_normal_cases[i].mmzName, &memFd);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_VB_GetBlock, ret %x!\n", ret);
        } else {
            ES_MEM_INFO("ES_VB_GetBlock fd %llx\n", memFd);
        }

        pVirAddr = ES_SYS_Mmap(memFd, blkSize, SYS_CACHE_MODE_CACHED);
        memset(pVirAddr, 0x55, blkSize);
        ES_SYS_MemFlushCache(memFd);

        /*do something....*/

        ES_SYS_Munmap(pVirAddr, blkSize);
        ES_MEM_INFO("ES_SYS_Munmap pVirAddr:%p\n", pVirAddr);

        ret = ES_VB_ReleaseBlock(memFd);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_VB_ReleaseBlock, ret %x!\n", ret);
        } else {
            ES_MEM_INFO("ES_VB_ReleaseBlock fd %llx\n", memFd);
        }

        /*destroy pool*/
        ret = ES_VB_Exit();
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_VB_Exit, ret %x!\n", ret);
            return ret;
        } else {
            ES_MEM_INFO("ES_VB_Exit SUCCESS!\n");
        }
    }

    return ret;
}

ES_S32 VB_mem_mod_case(void)
{
    ES_S32 i = 0, j = 0;
    VB_CONFIG_S vbConfig_in;
    VB_CONFIG_S vbConfig_out;
    VB_POOL_CONFIG_S *poolcfg;
    VB_UID_E enVbUid = VB_UID_VENC;
    ES_S32 ret;
    ES_S32 blkSize;
    ES_U64 memFd = 0;
    ES_VOID *pVirAddr;

    for (i = 0; i < sizeof(vb_normal_cases) / sizeof(struct case_vb_config_param_t); i++) {
        memset(&vbConfig_in, 0, sizeof(VB_CONFIG_S));
        memset(&vbConfig_out, 0, sizeof(VB_CONFIG_S));

        vbConfig_in.poolCnt = vb_normal_cases[i].poolCnt;
        blkSize = vb_normal_cases[i].blkSize;

        for (j = 0; j < vbConfig_in.poolCnt; j++) {
            poolcfg = &vbConfig_in.poolCfgs[j];
            poolcfg->blkSize = vb_normal_cases[i].blkSize;
            poolcfg->blkCnt = vb_normal_cases[i].blkCnt;
            poolcfg->enRemapMode = vb_normal_cases[i].enRemapMode;
            memcpy(poolcfg->mmzName, vb_normal_cases[i].mmzName, strlen(vb_normal_cases[i].mmzName));
        }

        ret = ES_VB_SetModPoolConfig(enVbUid, &vbConfig_in);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("SetModConfig, ret %x!\n", ret);
            return ret;
        }

        ret = ES_VB_GetModPoolConfig(enVbUid, &vbConfig_out);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_VB_GetModPoolConfig, ret %x!\n", ret);
            return ret;
        }

        for (j = 0; j < vbConfig_in.poolCnt; j++) {
            if ((vbConfig_in.poolCfgs[j].blkCnt != vbConfig_out.poolCfgs[j].blkCnt) ||
                (vbConfig_in.poolCfgs[j].blkSize != vbConfig_out.poolCfgs[j].blkSize) ||
                (vbConfig_in.poolCfgs[j].enRemapMode != vbConfig_out.poolCfgs[j].enRemapMode)) {
                ES_MEM_ERR("error:compare poolcfg failed!\n");
                return -1;
            }

            if (strlen(vbConfig_in.poolCfgs[j].mmzName)) {
                if (memcmp(vbConfig_in.poolCfgs[j].mmzName, vbConfig_out.poolCfgs[j].mmzName, ES_MAX_MMZ_NAME_LEN) !=
                    0) {
                    ES_MEM_ERR("error:compare poolcfg mmzName failed!\n");
                    return -1;
                }
            } else {
                if (memcmp(vbConfig_out.poolCfgs[j].mmzName, DEFAULT_MMZ_NAME, strlen(DEFAULT_MMZ_NAME)) != 0) {
                    ES_MEM_ERR("error:compare poolcfg mmzName(%s) failed!\n", vbConfig_out.poolCfgs[j].mmzName);
                    return -1;
                }
            }
        }

        ret = ES_VB_ModPoolInit(enVbUid);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("error:ES_VB_ModPoolInit, ret %x!\n", ret);
            return ret;
        }

        ret = ES_VB_GetModBlock(enVbUid, blkSize, vb_normal_cases[i].mmzName, &memFd);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_VB_GetModBlock, ret %x!\n", ret);
        } else {
            ES_MEM_INFO("ES_VB_GetModBlock fd %llx\n", memFd);
        }

        pVirAddr = ES_SYS_Mmap(memFd, blkSize, SYS_CACHE_MODE_CACHED);
        memset(pVirAddr, 0x55, blkSize);
        ES_SYS_MemFlushCache(memFd);

        /*do something....*/

        ES_SYS_Munmap(pVirAddr, blkSize);
        ES_MEM_INFO("ES_SYS_Munmap pVirAddr:%p\n", pVirAddr);

        ret = ES_VB_ReleaseBlock(memFd);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_VB_ReleaseBlock, ret %x!\n", ret);
        } else {
            ES_MEM_INFO("ES_VB_ReleaseBlock fd %llx\n", memFd);
        }

        /*destroy pool*/
        ret = ES_VB_ModPoolExit(enVbUid);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_VB_ModPoolExit, ret %x!\n", ret);
            return ret;
        } else {
            ES_MEM_INFO("ES_VB_ModPoolExit SUCCESS!\n");
        }
    }

    return ret;
}

ES_S32 VB_mem_mod_IOVA_case(void)
{
    ES_S32 i = 0, j = 0, k = 0;
    VB_CONFIG_S vbConfig_in;
    VB_CONFIG_S vbConfig_out;
    VB_POOL_CONFIG_S *poolcfg;
    VB_UID_E enVbUid = VB_UID_PRIVATE;
    VB_UID_E uid[] = {
        VB_UID_VENC,
        VB_UID_VDEC,
    };
    ES_S32 ret;
    ES_U64 memFd = 0;
    ES_U64 memFdRcv = 0;
    ES_VOID *pIOVA = NULL;
    ES_S32 blkSize;

    for (k = 0; k < sizeof(uid) / sizeof(uid[0]); k++) {
        enVbUid = uid[k];

        for (i = 0; i < sizeof(vb_normal_cases) / sizeof(struct case_vb_config_param_t); i++) {
            memset(&vbConfig_in, 0, sizeof(VB_CONFIG_S));
            memset(&vbConfig_out, 0, sizeof(VB_CONFIG_S));

            vbConfig_in.poolCnt = vb_normal_cases[i].poolCnt;
            blkSize = vb_normal_cases[i].blkSize;

            for (j = 0; j < vbConfig_in.poolCnt; j++) {
                poolcfg = &vbConfig_in.poolCfgs[j];
                poolcfg->blkSize = vb_normal_cases[i].blkSize;
                poolcfg->blkCnt = vb_normal_cases[i].blkCnt;
                poolcfg->enRemapMode = vb_normal_cases[i].enRemapMode;
                memcpy(poolcfg->mmzName, vb_normal_cases[i].mmzName, strlen(vb_normal_cases[i].mmzName));
            }

            ret = ES_VB_SetModPoolConfig(enVbUid, &vbConfig_in);
            if (ES_SUCCESS != ret) {
                ES_MEM_ERR("SetModConfig, ret %x!\n", ret);
                return ret;
            }

            ret = ES_VB_GetModPoolConfig(enVbUid, &vbConfig_out);
            if (ES_SUCCESS != ret) {
                ES_MEM_ERR("ES_VB_GetModPoolConfig, ret %x!\n", ret);
                return ret;
            }

            for (j = 0; j < vbConfig_in.poolCnt; j++) {
                if ((vbConfig_in.poolCfgs[j].blkCnt != vbConfig_out.poolCfgs[j].blkCnt) ||
                    (vbConfig_in.poolCfgs[j].blkSize != vbConfig_out.poolCfgs[j].blkSize) ||
                    (vbConfig_in.poolCfgs[j].enRemapMode != vbConfig_out.poolCfgs[j].enRemapMode)) {
                    ES_MEM_ERR("error:compare poolcfg failed!\n");
                    return -1;
                }

                if (strlen(vbConfig_in.poolCfgs[j].mmzName)) {
                    if (memcmp(vbConfig_in.poolCfgs[j].mmzName, vbConfig_out.poolCfgs[j].mmzName,
                               ES_MAX_MMZ_NAME_LEN) != 0) {
                        ES_MEM_ERR("error:compare poolcfg mmzName failed!\n");
                        return -1;
                    }
                } else {
                    if (memcmp(vbConfig_out.poolCfgs[j].mmzName, DEFAULT_MMZ_NAME, strlen(DEFAULT_MMZ_NAME)) != 0) {
                        ES_MEM_ERR("error:compare poolcfg mmzName(%s) failed!\n", vbConfig_out.poolCfgs[j].mmzName);
                        return -1;
                    }
                }
            }

            ret = ES_VB_ModPoolInit(enVbUid);
            if (ES_SUCCESS != ret) {
                ES_MEM_ERR("error:ES_VB_ModPoolInit, ret %x!\n", ret);
                return ret;
            }

            ret = ES_VB_GetModBlock(enVbUid, blkSize, vb_normal_cases[i].mmzName, &memFd);
            if (ES_SUCCESS != ret) {
                ES_MEM_ERR("ES_VB_GetModBlock, ret %x!\n", ret);
                return ret;
            } else {
                ES_MEM_INFO("ES_VB_GetModBlock memFd %llx\n", memFd);
            }

            ret = ES_VB_AllocIOVA(memFd, enVbUid, &pIOVA);
            if (ES_SUCCESS != ret) {
                ES_MEM_ERR("ES_VB_AllocIOVA, ret %x!\n", ret);
                goto IOVA_fail;
            } else {
                ES_MEM_INFO("ES_VB_AllocIOVA pIOVA %p\n", pIOVA);
            }

            ret = ES_VB_IOVA2Handle(enVbUid, pIOVA, &memFdRcv);
            if (ES_SUCCESS != ret) {
                ES_MEM_ERR("ES_VB_IOVA2Handle, ret %x!\n", ret);
            } else {
                if (memFdRcv != memFd) {
                    ES_MEM_ERR("memFdRcv(0x%llx) != memFd(0x%llx)\n", memFdRcv, memFd);
                }
            }

            ret = ES_VB_FreeIOVA(enVbUid, memFd);
            if (ES_SUCCESS != ret) {
                ES_MEM_ERR("ES_VB_FreeIOVA, ret %x!\n", ret);
            }

IOVA_fail:
            ret = ES_VB_ReleaseBlock(memFd);
            if (ES_SUCCESS != ret) {
                ES_MEM_ERR("ES_VB_ReleaseBlock, ret %x!\n", ret);
            } else {
                ES_MEM_INFO("ES_VB_ReleaseBlock fd %llx\n", memFd);
            }

            /*destroy pool*/
            ret = ES_VB_ModPoolExit(enVbUid);
            if (ES_SUCCESS != ret) {
                ES_MEM_ERR("ES_VB_ModPoolExit, ret %x!\n", ret);
            } else {
                ES_MEM_INFO("ES_VB_ModPoolExit SUCCESS!\n");
            }
        }
    }

    return 0;
}

VB_POOL_CONFIG_S private_pool_param[] = {
    {
        .blkSize = 4 * 1024 * 1024,
        .blkCnt = 20,
        .enRemapMode = SYS_CACHE_MODE_CACHED,
        .mmzName = "mmz_nid_0_part_0",
    },
    {
        .blkSize = 4 * 1024,
        .blkCnt = 64,
        .enRemapMode = SYS_CACHE_MODE_NOCACHE,
        .mmzName = "mmz_nid_0_part_0",
    },
};

ES_S32 private_pool_case(void)
{
    int i = 0;
    VB_POOL_CONFIG_S *pPoolcfg;
    ES_S32 ret;
    ES_U64 memFd = 0;
    VB_POOL poolId = ES_VB_INVALID_POOLID;
    VB_POOL poolIdGet = ES_VB_INVALID_POOLID;
    ES_VOID *pVirAddr;
    ES_S32 nid = -1;

    for (i = 0; i < sizeof(private_pool_param) / sizeof(VB_POOL_CONFIG_S); i++) {
        pPoolcfg = &private_pool_param[i];

        /*create pool*/
        ret = ES_VB_CreatePool(pPoolcfg, &poolId);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_VB_CreatePool, ret %x!\n", ret);
            return ret;
        }
        ES_MEM_INFO("ES_VB_CreatePool poolId:%d, blkSize=0x%llx\n", poolId, pPoolcfg->blkSize);

        /*get block*/
        ret = ES_VB_GetBlock(poolId, pPoolcfg->blkSize, pPoolcfg->mmzName, &memFd);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_VB_GetBlock, ret %x!\n", ret);
            return ret;
        }
        ES_MEM_INFO("ES_VB_GetBlock memFd:%llx\n", memFd);

        ret = ES_SYS_RetrieveMemNodeByFd(memFd, &nid);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_SYS_RetrieveMemNodeByFd failed, ret %x!\n", ret);
            ES_VB_ReleaseBlock(memFd);
            return ret;
        }
        ES_MEM_INFO("fd 0x%llx mem node is:%d\n", memFd, nid);

        pVirAddr = ES_SYS_Mmap(memFd, pPoolcfg->blkSize, SYS_CACHE_MODE_CACHED);
        ES_MEM_INFO("ES_SYS_Mmap memFd:%llx\n", memFd);

        memset(pVirAddr, 0x55, pPoolcfg->blkSize);

        ES_SYS_MemFlushCache(memFd);

        ret = ES_SYS_RetrieveMemNodeByVaddr(pVirAddr, &nid);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_SYS_RetrieveMemNodeByVaddr failed, ret %x!\n", ret);
            ES_SYS_Munmap(pVirAddr, pPoolcfg->blkSize);
            return ret;
        }
        ES_MEM_INFO("pVirAddr_src 0x%px mem node is:%d\n", pVirAddr, nid);

        ES_SYS_Munmap(pVirAddr, pPoolcfg->blkSize);
        ES_MEM_INFO("ES_SYS_Munmap pVirAddr:%p\n", pVirAddr);

        ES_VB_ReleaseBlock(memFd);
        ES_MEM_INFO("ES_VB_ReleaseBlock memFd:%llx\n", memFd);

        ret = ES_VB_GetBlock(poolId, pPoolcfg->blkSize, pPoolcfg->mmzName, &memFd);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_VB_GetBlock, ret %x!\n", ret);
            return ret;
        }
        ES_MEM_INFO("ES_VB_GetBlock memFd:%llx\n", memFd);

        ret = ES_VB_MmapPool(poolId);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_VB_MmapPool, ret %x!\n", ret);
            return ret;
        }
        ES_MEM_INFO("ES_VB_MmapPool poolId:%d\n", poolId);

        /*because ES_VB_MmapPool is uncached,so pVirAddr is uncached*/
        ret = ES_VB_GetBlockVirAddr(poolId, memFd, &pVirAddr);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_VB_GetBlockVirAddr, ret %x!\n", ret);
            return ret;
        }
        ES_MEM_INFO("ES_VB_GetBlockVirAddr pVirAddr:%p, blkSize=0x%llx\n", pVirAddr, pPoolcfg->blkSize);

        memset(pVirAddr, 0x55, pPoolcfg->blkSize);
        ES_MEM_INFO("memset pVirAddr:%p\n", pVirAddr);

        ret = ES_VB_MunmapPool(poolId);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_VB_MunmapPool fail, ret %x!\n", ret);
            return ret;
        }
        ES_MEM_INFO("ES_VB_MunmapPool\n");

        poolIdGet = ES_VB_Handle2PoolId(memFd);
        if (poolIdGet != poolId) {
            ES_MEM_ERR("ES_VB_Handle2PoolId fail, ret %x!\n", ret);
            return ret;
        }

        ES_VB_ReleaseBlock(memFd);
        ES_MEM_INFO("ES_VB_ReleaseBlock memFd:%llx\n", memFd);

        /*destroy pool*/
        ret = ES_VB_DestroyPool(poolId);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_VB_DestroyPool, ret %x!\n", ret);
            return ret;
        }
        ES_MEM_INFO("ES_VB_DestroyPool poolId:%d\n", poolId);
    }

    return ret;
}

ES_S32 VB_mem_split_test(void)
{
    ES_S32 ret = 0;
    ES_U64 memFd = 0;
    ES_U64 memFdSplit = 0;
    ES_VOID *pVirAddr = NULL;
    VB_POOL poolId = ES_VB_INVALID_POOLID;
    ES_U64 splitOffset = LEN_2M - LEN_4K;
    ES_U64 splitSize = LEN_2M;

    VB_POOL_CONFIG_S poolCfg = {
        .blkSize = LEN_4M,
        .blkCnt = 1,
        .enRemapMode = SYS_CACHE_MODE_NOCACHE,
        .mmzName = "mmz_nid_0_part_0",
    };

    /*create pool*/
    ret = ES_VB_CreatePool(&poolCfg, &poolId);
    if (ES_SUCCESS != ret) {
        ES_MEM_ERR("ES_VB_CreatePool, ret %x!\n", ret);
        return ret;
    }
    ES_MEM_INFO("ES_VB_CreatePool poolId %d\n", poolId);

    /*get block*/
    ret = ES_VB_GetBlock(poolId, poolCfg.blkSize, poolCfg.mmzName, &memFd);
    if (ES_SUCCESS != ret) {
        ES_MEM_ERR("ES_VB_GetBlock, ret %x!\n", ret);
        return ret;
    }

    ret = ES_SYS_SplitMemFd(memFd, splitOffset, splitSize, &memFdSplit);
    if (ES_SUCCESS != ret) {
        ES_MEM_ERR("ES_SYS_SplitMemFd failed, ret %x!\n", ret);

        goto release;
    }

    pVirAddr = ES_SYS_Mmap(memFdSplit, splitSize, SYS_CACHE_MODE_CACHED);
    if (!pVirAddr) {
        ES_MEM_ERR("ES_SYS_Mmap failed!\n");
        goto release;
    }
    ES_MEM_INFO("pVirAddr:0x%px memset!\n", pVirAddr);
    memset(pVirAddr, 0xbb, splitSize);

    ES_MEM_INFO("flush cache!\n");
    ret = ES_SYS_MemFlushCache(memFdSplit);
    if (ES_SUCCESS != ret) {
        ES_MEM_ERR("ES_SYS_MemFlushCache failed, ret %x!\n", ret);
        goto release;
    }

release:
    if (pVirAddr) {
        ret = ES_SYS_Munmap(pVirAddr, splitSize);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_SYS_Munmap failed, ret %x!\n", ret);
        }
    }

    if (memFdSplit) {
        ES_MEM_INFO("free memFd(%llx)\n", memFdSplit);
        ret = ES_VB_ReleaseBlock(memFdSplit);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_VB_ReleaseBlock failed, ret %x!\n", ret);
        }
    }

    if (memFd) {
        ES_MEM_INFO("free memFd(%llx)\n", memFd);
        ret = ES_VB_ReleaseBlock(memFd);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_VB_ReleaseBlock failed, ret %x!\n", ret);
        }
    }

    /*destroy pool*/
    ret = ES_VB_DestroyPool(poolId);
    if (ES_SUCCESS != ret) {
        ES_MEM_ERR("ES_VB_DestroyPool, ret %x!\n", ret);
    }

    return 0;
}

#define ftok_file_name "/etc/profile"
#define proj_id 0x36

ES_S32 transfer_memfd_process_case(void)
{
    ES_S32 ret;
    ES_U64 memFd;
    ES_U64 memFdRcv;
    ES_U64 memFdRcvSize;
    ES_CHAR *pVirAddr = NULL;
    ES_S32 shmid;
    void *shareMem = NULL;
    ES_S32 pid;
    ES_S32 i = 0;
    ES_S32 status;

    pid = fork();
    if (pid == 0) {  // child
        usleep(10000);
        key_t key = ftok(ftok_file_name, proj_id);
        if (key < 0) {
            ES_MEM_ERR("ftok error\n");
            return -1;
        }
        shmid = shmget(key, LEN_4K, IPC_CREAT);
        ES_MEM_INFO("key:%d shmid:%d\n", key, shmid);
        shareMem = shmat(shmid, NULL, 0);

        memFd = *((ES_U64 *)(shareMem));
        ES_MEM_INFO("recv: memFd:%llx\n", memFd);

        ret = ES_SYS_RcvMemFd(memFd, &memFdRcv);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_SYS_RcvMemFd failed, ret %x!\n", ret);
            goto free_fd_recv;
        }

        ret = ES_SYS_GetMemSize(memFdRcv, &memFdRcvSize);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_SYS_GetMemSize failed, ret %x!\n", ret);
            goto free_fd_recv;
        }
        ES_MEM_INFO("memFdRcvSize:%llx\n", memFdRcvSize);

        pVirAddr = ES_SYS_Mmap(memFdRcv, LEN_4K, SYS_CACHE_MODE_CACHED);
        if (!pVirAddr) {
            ES_MEM_ERR("ES_SYS_Mmap failed!\n");
            goto free_fd_recv;
        }

        memset(pVirAddr, 0xaa, LEN_4K);
        ES_MEM_INFO("memset share mem 0xaa\n");

		ret = ES_SYS_MemFlushCache(memFdRcv);
		if (ES_SUCCESS != ret) {
			ES_MEM_ERR("ES_SYS_MemFlushCache failed, ret %x!\n", ret);
			goto free_fd_recv;
		}

        if (pVirAddr) {
            ret = ES_SYS_Munmap(pVirAddr, LEN_4K);
            if (ES_SUCCESS != ret) {
                ES_MEM_ERR("ES_SYS_Munmap failed, ret %x!\n", ret);
            }
        }
free_fd_recv:
        if (memFdRcv) {
            ret = ES_SYS_MemFree(memFdRcv);
            if (ES_SUCCESS != ret) {
                ES_MEM_ERR("ES_SYS_MemFree failed, ret %x!\n", ret);
            }
        }

        if (shareMem) {
            shmdt(shareMem);
        }

        if (shmid != -1) {
            shmctl(shmid, IPC_RMID, NULL);
        }

        exit(0);
    } else {
        ES_S32 cmpOK = 1;

        key_t key = ftok(ftok_file_name, proj_id);
        if (key < 0) {
            ES_MEM_ERR("ftok error\n");
            return -1;
        }

        shmid = shmget(key, LEN_4K, IPC_CREAT | IPC_EXCL | 0644);
        if (shmid == -1) {
            ES_MEM_ERR("shmid == -1!\n");
            goto exit;
        }

        shareMem = shmat(shmid, NULL, 0);
        if (shareMem == NULL) {
            ES_MEM_ERR("shareMem is NULL!\n");
            goto exit;
        }
        ES_MEM_INFO("key:%d shmid:%d shareMem:0x%p\n", key, shmid, shareMem);

        ret = ES_SYS_MemAlloc(&memFd, SYS_CACHE_MODE_CACHED, "trans_a", "system", LEN_4K);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_SYS_MemAlloc failed, ret %x!\n", ret);
            goto exit;
        }

        ES_MEM_INFO("parent memFd:%llx\n", memFd);
        *((ES_U64 *)(shareMem)) = memFd;

        usleep(1000000);

        pVirAddr = ES_SYS_Mmap(memFd, LEN_4K, SYS_CACHE_MODE_CACHED);
        if (!pVirAddr) {
            ES_MEM_ERR("ES_SYS_Mmap failed!\n");
            goto exit;
        }
        ES_MEM_INFO("pVirAddr:0x%p\n", pVirAddr);

		ret = ES_SYS_MemFlushCache(memFd);
		if (ES_SUCCESS != ret) {
			ES_MEM_ERR("ES_SYS_MemFlushCache failed, ret %x!\n", ret);
			goto exit;
		}

        for (i = 0; i < LEN_4K; i++) {
            if (pVirAddr[i] != 0xaa) {
                ES_MEM_ERR("error data(0x%x)  in (%d)!\n", pVirAddr[i], i);
                cmpOK = 0;
                break;
            }
        }
        if (cmpOK == 1) {
            ES_MEM_INFO("compare share memory success!\n");
        }

exit:
        if (pVirAddr) {
            ret = ES_SYS_Munmap(pVirAddr, LEN_4K);
            if (ES_SUCCESS != ret) {
                ES_MEM_ERR("ES_SYS_Munmap failed, ret %x!\n", ret);
            }
        }

        if (memFd) {
            ret = ES_SYS_MemFree(memFd);
            if (ES_SUCCESS != ret) {
                ES_MEM_ERR("ES_SYS_MemFree failed, ret %x!\n", ret);
            }
        }
        if (shareMem) {
            shmdt(shareMem);
        }

        if (shmid != -1) {
            shmctl(shmid, IPC_RMID, NULL);
        }

        waitpid(-1, &status, WUNTRACED);
        if (WIFEXITED(status)) {
            ES_MEM_INFO("chilld return %d\n", WEXITSTATUS(status));
        } else {
            ES_MEM_INFO("chilld abnormal exit,%d\n", WEXITSTATUS(status));
        }

        return 0;
    }
}

static void *transfer_memfd_thread(void *pArg)
{
    ES_S32 ret = ES_SUCCESS;
    pthread_t threadId = pthread_self();
    ES_CHAR *pVirAddr;
    ES_U64 memFd;
    ES_U64 memFdRcv;

    memFd = *((ES_U64 *)pArg);
    ES_MEM_INFO("threadId=%ld memFd=0x%llx\n", threadId, memFd);

    ret = ES_SYS_RcvMemFd(memFd, &memFdRcv);
    if (ES_SUCCESS != ret) {
        ES_MEM_ERR("ES_SYS_RcvMemFd failed, ret %x!\n", ret);
        return (void *)-1;
    }

    ES_MEM_INFO("threadId=%ld memFdRcv=0x%llx\n", threadId, memFdRcv);

    pVirAddr = ES_SYS_Mmap(memFdRcv, LEN_4K, SYS_CACHE_MODE_NOCACHE);
    if (!pVirAddr) {
        ES_MEM_ERR("ES_SYS_Mmap failed!\n");
        return (void *)-1;
    }

    ES_MEM_INFO("pVirAddr:%p memset share mem 0xaa\n", pVirAddr);
    memset(pVirAddr, 0xaa, LEN_4K);

    if (pVirAddr) {
        ret = ES_SYS_Munmap(pVirAddr, LEN_4K);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_SYS_Munmap failed, ret %x!\n", ret);
        }
    }

    if (memFdRcv) {
        ret = ES_SYS_MemFree(memFdRcv);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_SYS_MemFree failed, ret %x!\n", ret);
        }
    }

    ES_MEM_INFO("testThreadFunc exit,threadId=%ld\n", threadId);

    return (void *)0;
}

ES_S32 transfer_memfd_thread_case(void)
{
    ES_S32 i = 0, j = 0;
    VB_CONFIG_S vbConfig_in;
    VB_POOL_CONFIG_S *poolcfg;
    ES_CHAR *mmzName = "mmz_nid_0_part_0";
    pthread_t thread;
    ES_CHAR *pVirAddr;
    ES_U64 memFd;
    ES_U64 blkSize = LEN_4K;
    ES_S32 ret;
    ES_S32 cmpOK = 0;

    memset(&vbConfig_in, 0, sizeof(VB_CONFIG_S));
    vbConfig_in.poolCnt = 1;
    for (j = 0; j < vbConfig_in.poolCnt; j++) {
        poolcfg = &vbConfig_in.poolCfgs[j];
        poolcfg->blkSize = blkSize;
        poolcfg->blkCnt = 10;
        poolcfg->enRemapMode = SYS_CACHE_MODE_NOCACHE;
        memcpy(poolcfg->mmzName, mmzName, strlen(mmzName));
    }

    ret = ES_VB_SetConfig(&vbConfig_in);
    if (ES_SUCCESS != ret) {
        ES_MEM_ERR("SetConfig, ret %x!\n", ret);
        return ret;
    }

    ret = ES_VB_Init();
    if (ES_SUCCESS != ret) {
        ES_MEM_ERR("error:ES_VB_Init, ret %x!\n", ret);
        return ret;
    }

    ret = ES_VB_GetBlock(ES_VB_INVALID_POOLID, blkSize, NULL, &memFd);
    if (ES_SUCCESS != ret) {
        ES_MEM_ERR("ES_VB_GetBlock, ret %x!\n", ret);
        return ret;
    }
    ES_MEM_INFO("memFd:0x%llx\n", memFd);

    pVirAddr = ES_SYS_Mmap(memFd, LEN_4K, SYS_CACHE_MODE_NOCACHE);
    if (!pVirAddr) {
        ES_MEM_ERR("ES_SYS_Mmap failed!\n");
        return -1;
    }
    ES_MEM_INFO("pVirAddr:0x%p\n", pVirAddr);

    memset(pVirAddr, 0x55, LEN_4K);

    pthread_create(&thread, NULL, transfer_memfd_thread, &memFd);
    pthread_detach(thread);

    sleep(3);

    for (i = 0; i < LEN_4K; i++) {
        if (pVirAddr[i] != 0xaa) {
            ES_MEM_ERR("error data(0x%x)  in (%d)!\n", pVirAddr[i], i);
            cmpOK = -1;
            break;
        }
    }

    if (pVirAddr) {
        ret = ES_SYS_Munmap(pVirAddr, LEN_4K);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_SYS_Munmap failed, ret %x!\n", ret);
        }
    }

    if (memFd) {
        ret = ES_SYS_MemFree(memFd);
        if (ES_SUCCESS != ret) {
            ES_MEM_ERR("ES_SYS_MemFree failed, ret %x!\n", ret);
        }
    }

    /*destroy pool*/
    ret = ES_VB_Exit();
    if (ES_SUCCESS != ret) {
        ES_MEM_ERR("ES_VB_Exit, ret %x!\n", ret);
        return ret;
    } else {
        ES_MEM_INFO("ES_VB_Exit SUCCESS!\n");
    }

    if (cmpOK == -1) {
        ES_MEM_ERR("VB_mem_multi_thread_test failed!\n");
    } else {
        ES_MEM_INFO("VB_mem_multi_thread_test SUCCESS!\n");
    }

    return cmpOK;
}

struct memory_case {
    const ES_CHAR *name;
    ES_S32 (*case_fn)(void);
};

static struct memory_case memory_cases[] = {
    {
        .name = "1.common memory case",
        .case_fn = comm_mem_case,
    },

    {
        .name = "2.VB memory case",
        .case_fn = VB_mem_case,
    },

    {
        .name = "3.VB mod memory case",
        .case_fn = VB_mem_mod_case,
    },
    {
        .name = "4.VB mod IOVA case",
        .case_fn = VB_mem_mod_IOVA_case,
    },
    {
        .name = "5.private pool case",
        .case_fn = private_pool_case,
    },
    {
        .name = "6.VB memory split case",
        .case_fn = VB_mem_split_test,
    },

    {
        .name = "7.transfer memfd in process case",
        .case_fn = transfer_memfd_process_case,
    },

    {
        .name = "8.transfer memfd in thread case",
        .case_fn = transfer_memfd_thread_case,
    },

};

/*
 * usage: memory_samples [case_num]
 * if case_num = 0, run call case
 */
ES_S32 main(ES_S32 argc, ES_CHAR *argv[])
{
    ES_S32 ret = 0;
    ES_S32 case_num = 0;
    ES_S32 arg = -1;

    ES_S32 case_total_num = sizeof(memory_cases) / sizeof(struct memory_case);

    ES_MEM_INFO("case_total_num=%d\n", case_total_num);

    if (argc > 1) {
        arg = str2int(argv[1]);
        if (arg > case_total_num) {
            printf("error:wrong input(%d)", arg);
            return -1;
        }
    }

    if (argc == 1 || arg == 0) {  // run all case
        for (case_num = 0; case_num < case_total_num; case_num++) {
            ES_MEM_INFO("-----------------case(%s)---------------\n", memory_cases[case_num].name);

            ret = memory_cases[case_num].case_fn();
            if (ret) {
                ES_MEM_ERR("-----------case(%s) test failed!--------\n\n", memory_cases[case_num].name);
            } else {
                ES_MEM_INFO("-----------case(%s) test success!--------\n\n", memory_cases[case_num].name);
            }
        }
    } else {
        case_num = arg - 1;

        ES_MEM_INFO("-----------------case(%s)---------------\n", memory_cases[case_num].name);

        ret = memory_cases[case_num].case_fn();
        if (ret) {
            ES_MEM_ERR("-----------case(%s) test failed!--------\n\n", memory_cases[case_num].name);
        } else {
            ES_MEM_INFO("-----------case(%s) test success!--------\n\n", memory_cases[case_num].name);
        }
    }

    return 0;
}
